package com.persagy.dmp.rwd.dic;

import cn.hutool.core.io.FileUtil;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.common.http.HttpUtils;
import com.persagy.dmp.rwd.dic.FuncidUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.File;
import java.io.FileInputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: yaoll
 * @date: 2020-11-30
 * @verison: 1.0
 */
@Slf4j
public class ExcelUtilsForEdit {

    public static Object parseCell(Cell cell, String type) {
        if (cell == null) {
            return null;
        }
        CellType cellType = cell.getCellTypeEnum();
        if ("string".equalsIgnoreCase(type)) {
            if (cellType == CellType.STRING) {
                String val = cell.getStringCellValue();
                return val == null ? null : val.trim();
            } else if (cellType == CellType.NUMERIC) {
                return cell.getNumericCellValue() + "";
            }
        } else if ("integer".equalsIgnoreCase(type)) {
            if (cellType == CellType.STRING) {
                String stringCellValue = cell.getStringCellValue();
                return stringCellValue == null ? null : Double.valueOf(stringCellValue).intValue();
            } else if (cellType == CellType.NUMERIC) {
                return Double.valueOf(cell.getNumericCellValue()).intValue();
            }
        } else if ("double".equalsIgnoreCase(type)) {
            if (cellType == CellType.STRING) {
                String stringCellValue = cell.getStringCellValue();
                return stringCellValue == null ? null : Double.valueOf(stringCellValue);
            } else if (cellType == CellType.NUMERIC) {
                return cell.getNumericCellValue();
            }
        } else if ("date".equalsIgnoreCase(type)) {
            if (cellType == CellType.NUMERIC) {
                Date dateCellValue = cell.getDateCellValue();
                return dateCellValue == null ? null : dateCellValue;
            } else if (cellType == CellType.STRING) {
                String val = cell.getStringCellValue();
                return val == null ? null : val.trim();
            }
        }
        return null;
    }


    private String basepath = "D:\\users\\数据字典v3.1.11\\数据字典附表(20210701)";

    @Test
    public void test1() throws Exception {
        String path = basepath + "/附表7-系统类对象分类属性点表";
        showPath(path);
    }

    public void showPath(String path) throws Exception {
        Map<String, String> dtType = new HashMap<>();
        dtType.put("SE", "强电");
        dtType.put("WE", "弱电");
        dtType.put("AC", "空调");
        dtType.put("FF", "消防");
        dtType.put("SP", "安防");
        dtType.put("WS", "给排水");
        dtType.put("CF", "土建");
        dtType.put("OT", "其他");
        List<String> names = listFile(path);
        if (names != null) {
            for (String name : names) {
//				System.out.println("----------------------");
                String fileName = path + "/" + name;
                FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
                Workbook workbook = new XSSFWorkbook(inputStream);
                int numberOfSheets = workbook.getNumberOfSheets();
                for (int i = 0; i < numberOfSheets; i++) {
                    Sheet sheet = workbook.getSheetAt(i);
                    String sheetName = sheet.getSheetName();
                    // funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
                    dtType.entrySet().forEach(item -> {
                        String key = item.getKey();
                        String value = item.getValue();
                        if (name.contains(value)) {
                            System.out.println("analysisSheetDataAndWriteToFile(\"" + key + sheetName.substring(0, 2) + "\", \"/附表7-系统类对象分类属性点表/" + name + "\", \"" + sheetName + "\",\"2\",systemCommonSeq);");
                            System.out.println("systemCommonSeq = new AtomicInteger(this.systemCommonSeq.intValue());");
                        }
                    });
                }
            }
        }
    }

    @Test
    public void test2() throws Exception {
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.1-强电专业设备", "SE");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.2-弱电专业设备", "WE");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.3-空调专业设备", "AC");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.4-给排水专业设备", "WS");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.5-消防专业设备", "FF");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.6-安防专业设备", "SP");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.7-土建精装专业设备", "CF");
        showPathForEquipment(basepath, "/附表8-设备类对象分类属性点表/" + "附表8.8-其他专业设备", "OT");
    }

    public void showPathForEquipment(String path, String pathName, String key) throws Exception {
        List<String> names = listFile(path + pathName);
        if (names != null) {
            for (String name : names) {
//				System.out.println("----------------------");
                String fileName = path + pathName + "/" + name;
                FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
                Workbook workbook = new XSSFWorkbook(inputStream);
                int numberOfSheets = workbook.getNumberOfSheets();
                for (int i = 0; i < numberOfSheets; i++) {
                    Sheet sheet = workbook.getSheetAt(i);
                    String sheetName = sheet.getSheetName();
                    // funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
                    System.out.println("analysisSheetDataAndWriteToFile(\"" + key + sheetName.substring(0, 4) + "\", \"" + pathName + "/" + name + "\", \"" + sheetName + "\",\"2\",equipCommonSeq);");
                    System.out.println("equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());");
                }
            }
        }
    }

    @Test
    public void test3() throws Exception {
        showPathForComponent(basepath + "/附表9-部件类对象分类属性点表");
    }

    public void showPathForComponent(String path) throws Exception {
        Map<String, String> dtType = new HashMap<>();
        dtType.put("SE", "强电");
        dtType.put("WE", "弱电");
        dtType.put("AC", "空调");
        dtType.put("FF", "消防");
        dtType.put("SP", "安防");
        dtType.put("WS", "给排水");
        dtType.put("CF", "土建");
        dtType.put("OT", "其他");
        List<String> names = listFile(path);
        if (names != null) {
            for (String name : names) {
//				System.out.println("----------------------");
                String fileName = path + "/" + name;
                FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
                Workbook workbook = new XSSFWorkbook(inputStream);
                int numberOfSheets = workbook.getNumberOfSheets();
                for (int i = 0; i < numberOfSheets; i++) {
                    Sheet sheet = workbook.getSheetAt(i);
                    String sheetName = sheet.getSheetName();
                    // funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
                    dtType.entrySet().forEach(item -> {
                        String key = item.getKey();
                        String value = item.getValue();
                        if (name.contains(value)) {
                            System.out.println("analysisSheetDataAndWriteToFile(\"" + key + sheetName.substring(0, 6) + "\", \"/附表9-部件类对象分类属性点表/" + name + "\", \"" + sheetName + "\",\"2\",componentCommonSeq);");
                            System.out.println("componentCommonSeq = new AtomicInteger(this.componentCommonSeq.intValue());");
                        }
                    });
                }
            }
        }
    }

    @Test
    public void test4() throws Exception {
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.1-五金耗材", "HC");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.2-工具耗材", "TC");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.3-覆盖材料耗材", "CM");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.4-油品及胶粘材料耗材", "OG");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.5-管路耗材", "PM");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.6-安全耗材", "SC");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.7-电器耗材", "EC");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.8-办公耗材", "OC");
        showPathForMaterialAndTool(basepath, "/附表11-耗材类对象分类属性点表/" + "附表11.9-生活耗材", "LC");
    }


    @Test
    public void test5() throws Exception {
        showPathForMaterialAndTool(basepath, "/附表12-工具类对象分类属性点表/" + "附表12.1-电动工具", "ET");
        showPathForMaterialAndTool(basepath, "/附表12-工具类对象分类属性点表/" + "附表12.2-气动工具", "PT");
        showPathForMaterialAndTool(basepath, "/附表12-工具类对象分类属性点表/" + "附表12.3-手动工具", "HT");
        showPathForMaterialAndTool(basepath, "/附表12-工具类对象分类属性点表/" + "附表12.4-其他工具", "OT");
    }

    @Test
    public void test6() throws Exception {
        showPathForTool(basepath + "/附表12-工具类对象分类属性点表");
    }

    public void showPathForMaterialAndTool(String path, String pathName, String key) throws Exception {
        List<String> names = listFile(path + pathName);
        if (names != null) {
            for (String name : names) {
//				System.out.println("----------------------");
                String fileName = path + pathName + "/" + name;
                FileInputStream inputStream = new FileInputStream(fileName);
//				System.out.println(fileName);
                Workbook workbook = new XSSFWorkbook(inputStream);
                int numberOfSheets = workbook.getNumberOfSheets();
                for (int i = 0; i < numberOfSheets; i++) {
                    Sheet sheet = workbook.getSheetAt(i);
                    String sheetName = sheet.getSheetName();
                    // funcids.addAll(readSheet("FFFA", "/2.2-设备设施类对象/2.2.3-系统/50-消防专业系统.xlsx", "FA火灾报警系统"));
                    System.out.println("analysisSheetDataAndWriteToFile(\"" + key+"-"+ sheetName + "\", \"" + pathName + "/" + name + "\", \"" + sheetName + "\",\"2\",equipCommonSeq,rwdDefClassMap);");
                    System.out.println("equipCommonSeq = new AtomicInteger(this.equipCommonSeq.intValue());");
                }
            }
        }
    }

    public void showPathForTool(String path) throws Exception {
        List<File> files = listFiles(path);
        for (File file : files) {
            String absolutePath = file.getAbsolutePath();
            System.out.println(absolutePath);


        }
    }


    private List<String> listFile(String path) {
        File file = new File(path);
        File[] files = file.listFiles();
        return Arrays.stream(files).map(File::getName).filter(item -> !item.startsWith("~")).collect(Collectors.toList());
    }

    private List<File> listFiles(String path) {
        List<File> files = FileUtil.loopFiles(path);
        return files.stream().filter(file -> !file.getName().startsWith("~")).collect(Collectors.toList());
    }
}
